'use strict'

exports.__esModule = true
exports.execute = exports.errorHandler = void 0

var _httpErrors = _interopRequireDefault(require('http-errors'))

var _graphql = require('graphql')

function _interopRequireDefault(obj) {
  return obj && obj.__esModule ? obj : { default: obj }
}

const errorHandler = () => async (ctx, next) => {
  try {
    await next()
  } catch (error) {
    let httpError = (0, _httpErrors.default)(error)
    if (!httpError.expose) httpError = (0, _httpErrors.default)()
    ctx.response.status = httpError.status
    if (!ctx.response.body) ctx.response.body = {}
    ctx.response.body.errors = Array.isArray(httpError.graphqlErrors)
      ? httpError.graphqlErrors.map(_graphql.formatError)
      : [(0, _graphql.formatError)(httpError)]
    ctx.app.emit('error', error, ctx)
  }
}

exports.errorHandler = errorHandler

const execute = options => {
  if (typeof options === 'undefined')
    throw (0, _httpErrors.default)(
      'GraphQL execute middleware options missing.'
    )
  if (!isPlainObject(options))
    throw (0, _httpErrors.default)(
      'GraphQL execute middleware options must be an object.'
    )
  const ALLOWED_OPTIONS = [
    'schema',
    'rootValue',
    'contextValue',
    'fieldResolver',
    'override'
  ]
  checkOptions(options, ALLOWED_OPTIONS, 'GraphQL execute middleware')
  if (typeof options.schema !== 'undefined') checkSchema(options.schema)
  if (
    typeof options.override !== 'undefined' &&
    typeof options.override !== 'function'
  )
    throw (0, _httpErrors.default)(
      'GraphQL execute middleware `override` option must be a function.'
    )
  return async (ctx, next) => {
    if (typeof ctx.request.body === 'undefined')
      throw (0, _httpErrors.default)('Request body missing.')
    if (!isPlainObject(ctx.request.body))
      throw (0, _httpErrors.default)(400, 'Request body must be a JSON object.')
    if (!('query' in ctx.request.body))
      throw (0, _httpErrors.default)(
        400,
        'GraphQL operation field `query` missing.'
      )
    let document

    try {
      document = (0, _graphql.parse)(
        new _graphql.Source(ctx.request.body.query)
      )
    } catch (error) {
      throw (0, _httpErrors.default)(
        400,
        `GraphQL query syntax error: ${error.message}`
      )
    }

    let optionsOverride = {}

    if (options.override) {
      optionsOverride = await options.override(ctx)
      if (!isPlainObject(optionsOverride))
        throw (0, _httpErrors.default)(
          'GraphQL execute middleware options must be an object, or an object promise.'
        )
      checkOptions(
        optionsOverride,
        ALLOWED_OPTIONS.filter(option => option !== 'override'),
        'GraphQL execute middleware `override` option return'
      )
      if (typeof optionsOverride.schema !== 'undefined')
        checkSchema(optionsOverride.schema)
    }

    const execute = {
      ...options,
      ...optionsOverride
    }
    const queryValidationErrors = (0, _graphql.validate)(
      execute.schema,
      document
    )
    if (queryValidationErrors.length)
      throw (0, _httpErrors.default)(400, 'GraphQL query validation errors.', {
        graphqlErrors: queryValidationErrors
      })
    let result

    try {
      result = await (0, _graphql.execute)({
        ...execute,
        document,
        variableValues: ctx.request.body.variables,
        operationName: ctx.request.body.operationName
      })
    } catch (error) {
      throw (0, _httpErrors.default)(
        400,
        `GraphQL operation field invalid: ${error.message}`
      )
    }

    if (result.data)
      ctx.response.body = {
        data: result.data
      }
    if (result.errors)
      throw (0, _httpErrors.default)(200, 'GraphQL errors.', {
        graphqlErrors: result.errors
      })
    ctx.response.status = 200
    await next()
  }
}

exports.execute = execute

const isPlainObject = value =>
  typeof value === 'object' && value !== null && !Array.isArray(value)

const checkOptions = (options, allowed, description) => {
  const invalid = Object.keys(options).filter(
    option => !allowed.includes(option)
  )
  if (invalid.length)
    throw (0, _httpErrors.default)(
      `${description} options invalid: \`${invalid.join('`, `')}\`.`
    )
}

const checkSchema = schema => {
  if (!(schema instanceof _graphql.GraphQLSchema))
    throw (0, _httpErrors.default)(
      'GraphQL schema is required and must be a `GraphQLSchema` instance.'
    )
  const schemaValidationErrors = (0, _graphql.validateSchema)(schema)
  if (schemaValidationErrors.length)
    throw (0, _httpErrors.default)('GraphQL schema validation errors.', {
      graphqlErrors: schemaValidationErrors
    })
}
